@@ -83,6 +83,7 @@ typedef struct {
 
 	TrackerSparqlStatement *content_query;
 	TrackerSparqlStatement *urn_query;
+	TrackerSparqlStatement *exists_query;
 
 	GTimer *timer;
 
@@ -252,22 +253,9 @@ crawler_check_directory_contents_cb (TrackerCrawler *crawler,
 								     parent, children);
 	}
 
-	if (process) {
-		TrackerDirectoryFlags parent_flags;
-		gboolean add_monitor;
-
-		tracker_indexing_tree_get_root (priv->indexing_tree,
-		                                parent, &parent_flags);
-
-		add_monitor = (parent_flags & TRACKER_DIRECTORY_FLAG_MONITOR) != 0;
-
-		if (add_monitor) {
-			tracker_monitor_add (priv->monitor, parent);
-		} else {
-			tracker_monitor_remove (priv->monitor, parent);
-		}
-	} else {
+	if (!process) {
 		priv->current_index_root->current_dir_content_filtered = TRUE;
+		tracker_monitor_remove (priv->monitor, parent);
 	}
 
 	return process;
@@ -529,8 +517,16 @@ crawl_directory_in_current_root (TrackerFileNotifier *notifier)
 		return FALSE;
 
 	while (!g_queue_is_empty (priv->current_index_root->pending_dirs)) {
+		TrackerDirectoryFlags flags;
+
 		directory = g_queue_pop_head (priv->current_index_root->pending_dirs);
-		priv->current_index_root->current_dir = directory;
+		g_set_object (&priv->current_index_root->current_dir, directory);
+
+		tracker_indexing_tree_get_root (priv->indexing_tree,
+		                                directory, &flags);
+
+		if ((flags & TRACKER_DIRECTORY_FLAG_MONITOR) != 0)
+			tracker_monitor_add (priv->monitor, directory);
 
 		/* Begin crawling the directory non-recursively.
 		 *
@@ -545,9 +541,11 @@ crawl_directory_in_current_root (TrackerFileNotifier *notifier)
 		if (tracker_crawler_start (priv->crawler,
 		                           directory,
 		                           priv->current_index_root->flags)) {
+			g_object_unref (directory);
 			return TRUE;
 		}
 
+		tracker_monitor_remove (priv->monitor, directory);
 		g_object_unref (directory);
 	}
 
@@ -559,25 +557,18 @@ finish_current_directory (TrackerFileNotifier *notifier,
                           gboolean             interrupted)
 {
 	TrackerFileNotifierPrivate *priv;
-	GFile *directory;
 
 	priv = tracker_file_notifier_get_instance_private (notifier);
-	directory = priv->current_index_root->current_dir;
-	priv->current_index_root->current_dir = NULL;
-	priv->current_index_root->current_dir_content_filtered = FALSE;
 
-	if (directory) {
-		/* If crawling was interrupted, we take all collected info as invalid.
-		 * Otherwise we dispose regular files here, only directories are
-		 * cached once crawling has completed.
-		 */
-		tracker_file_system_forget_files (priv->file_system,
-						  directory,
-						  interrupted ?
-						  G_FILE_TYPE_UNKNOWN :
-						  G_FILE_TYPE_REGULAR);
-		g_object_unref (directory);
-	}
+	/* If crawling was interrupted, we take all collected info as invalid.
+	 * Otherwise we dispose regular files here, only directories are
+	 * cached once crawling has completed.
+	 */
+	tracker_file_system_forget_files (priv->file_system,
+	                                  priv->current_index_root->root,
+	                                  interrupted ?
+	                                  G_FILE_TYPE_UNKNOWN :
+	                                  G_FILE_TYPE_REGULAR);
 
 	if (interrupted || !crawl_directory_in_current_root (notifier)) {
 		/* No more directories left to be crawled in the current
@@ -707,14 +698,40 @@ sparql_urn_ensure_statement (TrackerFileNotifier  *notifier,
 		tracker_sparql_connection_query_statement (priv->connection,
 		                                           "SELECT ?ie "
 		                                           "{"
-		                                           "  ~file a nfo:FileDataObject ;"
-		                                           "        nie:interpretedAs ?ie ."
+		                                           "  GRAPH tracker:FileSystem {"
+		                                           "    ~file a nfo:FileDataObject ;"
+		                                           "          nie:interpretedAs ?ie ."
+							   "  }"
 		                                           "}",
 		                                           priv->cancellable,
 		                                           error);
 	return priv->urn_query;
 }
 
+static TrackerSparqlStatement *
+sparql_exists_ensure_statement (TrackerFileNotifier  *notifier,
+				GError              **error)
+{
+	TrackerFileNotifierPrivate *priv;
+
+	priv = tracker_file_notifier_get_instance_private (notifier);
+
+	if (priv->exists_query)
+		return priv->exists_query;
+
+	priv->exists_query =
+		tracker_sparql_connection_query_statement (priv->connection,
+		                                           "ASK "
+		                                           "{"
+		                                           "  GRAPH tracker:FileSystem {"
+		                                           "    ~file a nfo:FileDataObject ."
+							   "  }"
+		                                           "}",
+		                                           priv->cancellable,
+		                                           error);
+	return priv->exists_query;
+}
+
 static void
 query_execute_cb (TrackerSparqlStatement *statement,
                   GAsyncResult           *res,
@@ -898,7 +915,8 @@ notifier_queue_root (TrackerFileNotifier   *notifier,
 		priv->pending_index_roots = g_list_append (priv->pending_index_roots, data);
 	}
 
-	notifier_check_next_root (notifier);
+	if (!priv->current_index_root)
+		notifier_check_next_root (notifier);
 }
 
 /* This function ensures to issue ::file-created for all
@@ -1063,6 +1081,12 @@ monitor_item_updated_cb (TrackerMonitor *monitor,
 	/* Fetch the interned copy */
 	canonical = tracker_file_system_get_file (priv->file_system,
 	                                          file, file_type, NULL);
+
+	if (is_directory) {
+		/* Ensure the folder iri is cached */
+		tracker_file_notifier_get_file_iri (notifier, canonical, TRUE);
+	}
+
 	g_signal_emit (notifier, signals[FILE_UPDATED], 0, canonical, FALSE);
 
 	if (!is_directory) {
@@ -1529,6 +1553,7 @@ tracker_file_notifier_finalize (GObject *object)
 
 	g_clear_object (&priv->content_query);
 	g_clear_object (&priv->urn_query);
+	g_clear_object (&priv->exists_query);
 
 	g_object_unref (priv->crawler);
 	g_object_unref (priv->monitor);
@@ -1946,6 +1971,66 @@ tracker_file_notifier_get_file_iri (TrackerFileNotifier *notifier,
 	return iri;
 }
 
+gboolean
+tracker_file_notifier_query_file_exists (TrackerFileNotifier *notifier,
+					 GFile               *file)
+{
+	TrackerFileNotifierPrivate *priv;
+	GFile *canonical;
+	gboolean found;
+
+	g_return_val_if_fail (TRACKER_IS_FILE_NOTIFIER (notifier), FALSE);
+	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+	priv = tracker_file_notifier_get_instance_private (notifier);
+
+	if (G_UNLIKELY (priv->connection == NULL)) {
+		return FALSE;
+	}
+
+	canonical = tracker_file_system_get_file (priv->file_system,
+	                                          file,
+	                                          G_FILE_TYPE_REGULAR,
+	                                          NULL);
+	if (!canonical) {
+		return FALSE;
+	}
+
+	found = tracker_file_system_get_property_full (priv->file_system,
+	                                               canonical,
+						       quark_property_mimetype,
+						       NULL);
+
+	if (!found) {
+		TrackerSparqlCursor *cursor;
+		TrackerSparqlStatement *statement;
+		gchar *uri;
+
+		/* Fetch data for this file synchronously */
+		statement = sparql_exists_ensure_statement (notifier, NULL);
+		if (!statement)
+			return FALSE;
+
+		uri = g_file_get_uri (file);
+		tracker_sparql_statement_bind_string (statement, "file", uri);
+		g_free (uri);
+
+		cursor = tracker_sparql_statement_execute (statement, NULL, NULL);
+		if (!cursor)
+			return FALSE;
+
+		if (!tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			g_object_unref (cursor);
+			return FALSE;
+		}
+
+		found = tracker_sparql_cursor_get_boolean (cursor, 0);
+		g_object_unref (cursor);
+	}
+
+	return found;
+}
+
 static gboolean
 file_notifier_invalidate_file_iri_foreach (GFile    *file,
                                            gpointer  user_data)
