@@ -267,6 +267,22 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 		ed = g_new0 (TrackerExifData, 1);
 	}
 
+	if (!xd) {
+		gchar *sidecar = NULL;
+
+		xd = tracker_xmp_new_from_sidecar (file, &sidecar);
+
+		if (sidecar) {
+			TrackerResource *sidecar_resource;
+
+			sidecar_resource = tracker_resource_new (sidecar);
+			tracker_resource_add_uri (sidecar_resource, "rdf:type", "nfo:FileDataObject");
+			tracker_resource_add_relation (sidecar_resource, "nie:interpretedAs", metadata);
+
+			tracker_resource_add_take_relation (metadata, "nie:isStoredAs", sidecar_resource);
+		}
+	}
+
 	if (!xd) {
 		xd = g_new0 (TrackerXmpData, 1);
 	}
