@@ -23,15 +23,6 @@
 
 #include <glib.h>
 
-#include <gsf/gsf.h>
-#include <gsf/gsf-doc-meta-data.h>
-#include <gsf/gsf-infile.h>
-#include <gsf/gsf-infile-msole.h>
-#include <gsf/gsf-input-stdio.h>
-#include <gsf/gsf-msole-utils.h>
-#include <gsf/gsf-utils.h>
-#include <gsf/gsf-infile-zip.h>
-
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-main.h"
